package
Version:
v0.0.0-...-f92b857
Opens a new window with list of versions in this module.
Published: Nov 4, 2025
License: MIT
Opens a new window with license information.
Imports: 6
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
Error build error
const (
ErrNotPointer Error = iota
ErrNotStruct
ErrNotImplemented
)
error definitions
Handler edifact.Handler implementation to build arbitrary data structures.
func NewHandler(target interface{}) *Handler
NewHandler creates a new handler.
func (*Handler) Handle ¶
Handle edifact.Handler implementation.
Source Files
¶
Click to show internal directories.
Click to hide internal directories.